
// constructor -
// take docid and pull xml from server, then turn it into a JS object
// MarcRecord.marcObj - JS object of marc data
// MarcRecord.xml - marc XML data pulled from server
function MarcRecord(docid) {
	var xml = null;
	var cgi = new CGI();
	var xhr = buildXMLRequest();
	var proto = (location.href.match(/^[Hh][Tt][Tt][Pp][Ss]:/))?"https://":"http://";
	
	xhr.open("GET",proto+cgi.server_name+"/opac/extras/supercat/retrieve/marcxml/record/"+docid, false);
	xhr.send(null);
	if(xhr.responseXML && xhr.responseXML.documentElement && xhr.status == 200) xml = xhr.responseXML.documentElement;
	//this.toString = function() { return this.marcObj.toString(); };
	this.xml = xml;
	this.marcObj = this.XML2js(this.xml);
}

/* fields - string with 1 field or array of fields to process from the marc record
   optional args (as object):
     container  - string with dom id of container to fill with rendered data - if not present, MarcRecord.render() will return rendered code as a string instead
     label      - string with 1 dom id or array of dom ids to unhide if marc data is present
	 searchType - will turn each subfield into a searchable link using the specified search type (e.g. STYPE_KEYWORD, found in config.js)
	 showAGR    - boolean value, set to true if marc field 880 data is needed for each matching field
	 showOnly   - string with 1 subfield or array of subfields to process
	 skipCodes  - string with 1 subfield or array of subfields to skip over
	 unlink     - only valid if searchType is present - string with 1 subfield or array of subfields to NOT turn into links (will still return its data though)
	 linkText   - string with 1 subfield or ordered array of subfields to check for link text, only valid if linkSrc is present, resulting fields will be turned into links using linkSrc as the href
	 linkSrc    - string with 1 subfield that contains the link href for linkText
	 fGlue      - if multiple matching fields exist in the marc record, join them together using fGlue.joinStr string, 
	                use fGlue.startStr to add code to the start of the set of matched fields
				    use fGlue.endStr to add code to the end of the set of matched fields
	 sGlue      - if mutiple subfields exist for each matched field, join them together using sGlue.joinStr sting,
	                use sGlue.startStr to add code to the start of each match field
					use sGlue.endStr to add code to the end of each field
	
author example:
	MarcRecord.render(
		["100", "110", "111"],  // fields to look for
		{
			"container":"rdetail_authors_inner", // DOM element to dump resulting code into (will clear the element's contents)
			"searchType":STYPE_AUTHOR,			 // make each subfield a search link using search type STYPE_AUTHOR
			"label":["rdetail_authors_link", "authors_lbl"],  // unHide these DOM elements if any matching fields ("100", "110", "111") are found in the marc record
			"sGlue":{"joinStr":" ", "endStr":"<br />"},       // join each subfield together with a space and add a <br /> element to the end
			"showAGR":true,                      // if matching 880 data is present, show it - sGlue treats it as if it were a new field
			"unlink":"d"                         // do NOT turn subfield "d" into a searchable link
		}
	);
*/
MarcRecord.prototype.render = function(fields, args) {
  if(!fields) return;
  var retVal = "";
  var returnStr = false;
  args = args || {};
  if(typeof fields=="string") fields = [fields];
  if(typeof args.label=="undefined") args.label = [];
  if(typeof args.label=="string") args.label = [args.label];
  if(typeof args.showOnly=="undefined") args.showOnly = [];
  if(typeof args.showOnly=="string") args.showOnly = [args.showOnly];
  if(typeof args.skipCodes=="undefined") args.skipCodes = [];
  if(typeof args.skipCodes=="string") args.skipCodes = [args.skipCodes];
  if(typeof args.unlink=="undefined") args.unlink = [];
  if(typeof args.unlink=="string") args.unlink = [args.unlink];
  if(typeof args.linkText=="undefined") args.linkText = [];
  if(typeof args.linkText=="string") args.linkText = [args.linkText];
  var cont = typeof args.container == "undefined" ? null : $(args.container);
  if(!cont) returnStr = true;
  
  args.fGlue = args.fGlue || {};
  args.sGlue = args.sGlue || {};
  args.fGlue.startStr = args.fGlue.startStr || "";
  args.fGlue.endStr = args.fGlue.endStr || "";
  args.sGlue.startStr = args.sGlue.startStr || "";
  args.sGlue.endStr = args.sGlue.endStr || "";
  args.fGlue.joinStr = args.fGlue.joinStr || "";
  args.sGlue.joinStr = args.sGlue.joinStr || "";
  
  for(var i in fields) {
    var f = this.marcObj[fields[i]];
    if(!f) continue;
	for(var n in args.label) unHideMe($(args.label[n]));
	
	if(args.linkSrc) args.linkText.push(args.linkSrc);
    f.args = {};
    f.args.showAGR = args.showAGR || false;
    f.args.skipCodes = args.skipCodes;
	f.args.showOnly = args.showOnly;
	f.args.unlink = args.unlink;
	f.args.linkText = args.linkText;
	f.args.linkSrc = args.linkSrc || null;
	f.args.searchType = args.searchType || null;
    f.args.sGlue = args.sGlue;
  
    var str = f.join(args.fGlue.joinStr);
    str = args.fGlue.startStr + str + args.fGlue.endStr;
  
    if(returnStr) retVal+=str;
    else cont.innerHTML += str;
  }
  
  if(returnStr) return retVal;
};

// wrap each item in the array with an anchor tag for searching
// excluding those subfields specified in 'codes'
MarcRecord.prototype.buildSearchStr = function(arr, args, codes) {
  for(var i in arr) {
	var skip = false;
	for(var n in args.unlink)
	  if(args.unlink[n]==codes[i]) skip = true;
	if(skip) continue;
	var args2 = {};
	if( SHOW_MR_DEFAULT || findCurrentPage() == MRESULT ) {
		args2.page = MRESULT;
	} else {
		args2.page = RRESULT;
		args2[PARAM_RTYPE] = args.searchType;
	}
	args2[PARAM_OFFSET] = 0;
	args2[PARAM_TERM] = arr[i];
	args2[PARAM_STYPE] = args.searchType;
	arr[i] = "<a href='"+buildOPACLink(args2).replace("'", "%27")+"'>"+arr[i]+"</a>";
  }
};

// if field 880 exists, append its data to matching normal field
// AGR = Alternate Graphic Representation -
// typically used to display foreign languages
MarcRecord.prototype.getAGRdata = function(field, args) {
  var str = "";
  var val = parseInt(field.split("-")[1]);
  field = this.marcObj["880"][--val];
  field.args = args;
  str += field;
  return str;
};

// toString function override for subfield objects
MarcRecord.prototype.marcObjToStr = function() {
  var str = "";
  var str2 = ""; // AGR data
  var args = this.args || this.master.marcObj[this.field].args || {"searchType":null, "linkSrc":null, "linkText":[], "unlink":[], "showOnly":[], "skipCodes":[], "showAGR":false, "sGlue":{"joinStr":"", "endStr":"", "startStr":""}};
  var arr = [];
  var subs = [];
  var found = false;
  for(var i in this.codes) {
	var c = this.codes[i];
	var skipCode = false;
	for(var n in args.skipCodes) if(c==args.skipCodes[n]) skipCode = true;
	if(skipCode) continue;
	if(c=="6" && this[c][0].indexOf("880")>=0 && args.showAGR) str2+=this.master.getAGRdata(this[c][0], args);
	if(c=="0" || c=="2" || c=="3" || c=="4" || c=="5" || c=="6" || c=="8") continue;
	if(args.linkSrc && this[args.linkSrc]) {
	  var src = this[args.linkSrc][0];
	  for(var n in args.linkText) {
		if(this[args.linkText[n]]) { arr.push("<a target='_blank' href='"+encodeURI(src)+"'>"+this[args.linkText[n]].join(args.sGlue.joinStr)+"</a>"); found = true; break; }
	  }
	  if(found) break;
	}
	subs.push(c);
	if(args.showOnly.length) 
	  for(var n in args.showOnly) { 
	    if(args.showOnly[n]==c) arr = arr.concat(this[c]);
	  }
	else arr = arr.concat(this[c]);
  }
  
  if(args.searchType) this.master.buildSearchStr(arr, args, subs);
  str += arr.join(args.sGlue.joinStr);
  str = args.sGlue.startStr + str + args.sGlue.endStr;
  return str+str2;
};

/* marc XML to JS object conversion function
   resulting object will look similar to:
   MarcRecord.marcObj = {
	 "245":[
       {"a":["Title of item"], "c":["More title data"]},
	   {"a":["Another 245 field"], "c":["More title data"]}
     ],
	 
	 "100":[
	   {"a":["Author", "Author last name"], "b":["Author life span"]},
	   {"a":["Author 2", "Author2 last name"], "b":["Author2 life span"]}
     ]
   }
*/
MarcRecord.prototype.XML2js = function(parent) {
  var marc = {};
  var nodes = parent.childNodes || [];
  // field-level (and master) object augmentation
  //marc.toString = this.marcObjToStr2;
  marc.master = this; // link back to global parent object

  for(var i=0; i<nodes.length; i++) {
	var obj = nodes[i];
	if(obj.hasChildNodes()) {
	  var tag = (obj.nodeName=="datafield")?obj.getAttribute("tag"):null;
	  if(obj.nodeName=="datafield") {
		if(!marc[tag]) marc[tag] = [];
		var subfields = {"codes":[], "order":[]};
		// subfield object augmentation
		subfields.toString = this.marcObjToStr;
		subfields.master = this; // link back to global parent object
		subfields.field = tag;
		
		var hasSubs = {};
		for(var n=0; n<obj.childNodes.length; n++) {
		  var child = obj.childNodes[n];
		  if(child.nodeName=="subfield") {
			var code = child.getAttribute("code");
			if(!subfields[code]) subfields[code] = [];
			if(child && child.childNodes && child.childNodes[0])
				subfields[code].push(child.childNodes[0].textContent || child.childNodes[0].text);  // IE check
			if(!hasSubs[code]) subfields.codes.push(code);
			subfields.order.push(code);
			hasSubs[code] = true;
		  }
		}
		marc[tag].push(subfields);
	  }
	  if(!tag) marc = this.XML2js(obj);
	}
  }
  return marc;
};
